# -*- coding: utf-8 -*-
import sys
from subprocess import Popen, PIPE
from os.path import abspath, dirname, join
from unittest import TestCase

SKOOLKIT_DIR = abspath(dirname(dirname(__file__)))
sys.path.insert(0, SKOOLKIT_DIR)
SKOOL2ASM = join(SKOOLKIT_DIR, 'skool2asm.py')
SKOOL2CTL = join(SKOOLKIT_DIR, 'skool2ctl.py')
SKOOL2HTML = join(SKOOLKIT_DIR, 'skool2html.py')
SNA2SKOOL = join(SKOOLKIT_DIR, 'sna2skool.py')

class SkoolKitTestCase(TestCase):
    def assert_no_errors(self, error, cmdline, msg=None):
        if msg is None:
            msg = "Error(s) while running '%s':\n-----\n%s\n-----\n" % (cmdline, error)
        self.assertTrue(len(error) == 0, msg)

    def assert_output_equal(self, output, expected, prefixes=True):
        for i, line in enumerate(expected):
            if i < len(output):
                if prefixes:
                    self.assertEqual(output[i][:len(line)], line)
                else:
                    self.assertEqual(output[i], line)
        if len(output) > len(expected):
            self.fail("Unexpected extra line in output: '%s'" % output[len(expected)])
        elif len(expected) > len(output):
            self.fail("Expected lines missing from output, starting with: '%s'" % expected[len(output)])

    def to_lines(self, text, strip_cr):
        # Use rstrip() to remove '\r' characters (useful on Windows)
        if strip_cr:
            lines = [line.rstrip() for line in text.split('\n')]
        else:
            lines = text.split('\n')
        if lines[-1] == '':
            lines.pop()
        return lines

    def run_cmdline(self, argv, out_lines=True, err_lines=False, strip_cr=True):
        p = Popen(argv, stdout=PIPE, stderr=PIPE)
        out, err = p.communicate()
        if out_lines:
            out = self.to_lines(out, strip_cr)
        if err_lines:
            err = self.to_lines(err, strip_cr)
        return ' '.join(argv), out, err

    def run_cmd(self, cmd, out_lines=True, err_lines=False, strip_cr=True):
        return self.run_cmdline(cmd.split(), out_lines, err_lines, strip_cr)

    def _run_skoolkit_cmd(self, cmd, args, out_lines, err_lines, strip_cr):
        cmdline = [cmd] + args.split()
        if sys.platform == 'win32':
            cmdline.insert(0, 'python')
        return self.run_cmdline(cmdline, out_lines, err_lines, strip_cr)

    def run_skool2asm(self, args='', out_lines=True, err_lines=False, strip_cr=True):
        return self._run_skoolkit_cmd(SKOOL2ASM, args, out_lines, err_lines, strip_cr)

    def run_skool2ctl(self, args='', out_lines=True, err_lines=False, strip_cr=True):
        return self._run_skoolkit_cmd(SKOOL2CTL, args, out_lines, err_lines, strip_cr)

    def run_skool2html(self, args='', out_lines=True, err_lines=False, strip_cr=True):
        return self._run_skoolkit_cmd(SKOOL2HTML, args, out_lines, err_lines, strip_cr)

    def run_sna2skool(self, args='', out_lines=True, err_lines=False, strip_cr=True):
        return self._run_skoolkit_cmd(SNA2SKOOL, args, out_lines, err_lines, strip_cr)
